import requests
import yaml
import os
from pathlib import Path


def get_outside_ip():
    ip = ""
    try:
        ip = requests.get("http://ifconfig.me/ip", timeout=5).text.strip()
    except Exception as e:
        pass
    return ip


def is_encomm_char(_str):
    if not _str.encode('UTF-8').isalpha():
        return False

    return True


def get_current_path():
    aiapi_dir = os.path.join(Path.home(), ".aiapi")
    if os.path.exists(aiapi_dir):
        return aiapi_dir
    else:
        path = Path(f"{aiapi_dir}")
        path.mkdir(parents=True, exist_ok=True)
        return aiapi_dir


def read_app_yaml(directory_path, app_type):
    openapi_yaml_file_path = directory_path
    if app_type is not None:
        if directory_path.find(".yaml") == -1:
            openapi_yaml_file_path = os.path.join(
                directory_path, f"template_{app_type}.yaml")
    try:
        with open(openapi_yaml_file_path, "r", encoding="utf-8") as f:
            openapi_yaml = f.read()
        openapi_yaml = yaml.safe_load(openapi_yaml)
    except Exception as e:
        print(e)
        return ""
    return openapi_yaml


def write_new_yaml(yaml_file, app_type):
    if app_type != "openai":
        return
    with open(yaml_file, "r", encoding="utf-8") as f:
        openapi_yaml = f.read()
    openapi_yaml = yaml.safe_load(openapi_yaml)
    openapi_yaml['openapi'] = "3.0.1"
    del openapi_yaml['swagger']
    with open(f"{os.getcwd()}{os.sep}aiapi.yaml", "w", encoding="utf") as f:
        yaml.dump(openapi_yaml, stream=f, allow_unicode=True)



import argparse
import getpass
import sys
from .app.version import version
from .app.new_app import *


def echo_help():
    # 显示一个帮助信息
    _log = f'''
  _____ _____ _____ _____ _____ _____ _____ _____ 
 / ____|_   _|  __ \_   _|_   _|_   _|_   _|_   _|
| (___   | | | |__) || |   | |   | |   | |   | |  
 \___ \  | | |  ___/ | |   | |   | |   | |   | |  
 ____) |_| |_| |    _| |_ _| |_ _| |_ _| |_ _| |_ 
|_____/|_____|_|   |_____|_____|_____|_____|_____|

/* 专业应用托管服务平台 */

本地客户端版本: {version}, 最新客户端版本: {get_cli_server_version()}:

用法: sipiiiii [选项]

  显示帮助信息:
    sipiiiii -h 或 --help

  账号管理:
    登录你的sipiiiii账号, 登录后token有效期为3天:
      sipiiiii -L 或 --login

    注册一个sipiiiii账号:
      sipiiiii -R 或 --register

    邮件激活你的sipiiiii账号, 每个激活后的账号将赠送10积分:
      sipiiiii -A 或 --activate

    查看你的剩余积分:
      sipiiiii -t 或 --tally

  应用部署:
    运行应用(非部署)，在应用目录下直接运行:
      sipiiiii -d 或 --dev

    本地化部署应用，不需要上传代码到我们的部署服务器，在本地部署一个应用，并且我们提供一个二级域名解析到你的外网IP
    每部署一个本地应用将消耗5积分, 一个应用名只扣除一次积分:
      sipiiiii -LD <app name> 或 --localdepl <app name>
      sipiiiii -LD <test.yaml> 或 --localdepl <test.yaml>

    部署你的应用，将在我们的部署服务器中运行，并且我们提供一个二级域名解析
    每部署一个远程应用将消耗20积分, 一个应用名只扣除一次积分:
      sipiiiii -D 或 --deployed

  应用生成:
    根据 yaml 模版和选项创建一个应用框架
    有三种方式创建应用
    1) 根据自己提供的yaml文件创建应用
    2) 使用aiapi的模版创建应用
    3) 使用向导模式创建应用
      sipiiiii -n <test.yaml> 或 --new <test.yaml>

  应用管理:
    列出你的应用列表:
      sipiiiii -l 或 --list

    查看你的应用详情:
      sipiiiii -i <app name> 或 --info <app name>
      sipiiiii -i <test.yaml> 或 --info <test.yaml>

    停止你正在运行的应用(非本地部署):
      sipiiiii -s <app name> 或 --stop <app name>
      sipiiiii -s <test.yaml> 或 --stop <test.yaml>

    重启你的应用(非本地部署):
      sipiiiii -r <app name> 或 --restart <app name>
      sipiiiii -r <test.yaml> 或 --restart <test.yaml>
      
  示例:
    使用注册邮箱和密码进行登录
    sipiiiii -L 
    创建一个新的应用, 创建后编写相关的业务代码
    sipiiiii -n 
    进入到应用目录后, 进行部署
    sipiiiii -D
    查看应用详情信息 
    sipiiiii -i 应用名

'''
    return _log


def main():
    parser = argparse.ArgumentParser(description='说明', add_help=False)
    parser.add_argument('-h', '--help', help='说明文档',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-n', '--new', help='根据应用模版创建应用',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-d', '--dev', help='运行应用(非部署)',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-D', '--deployed', help='部署应用',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-L', '--login', help='登录',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-R', '--register', help='注册',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-A', '--activate', help='激活账号',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-l', '--list', help='获取你的应用列表',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-t', '--tally', help='查看积分',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-i', '--info', help='显示应用信息',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-s', '--stop', help='停止应用',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-r', '--restart', help='重启应用',
                        required=False, nargs='?', const='default_value')
    parser.add_argument('-LD', '--localdepl', help='本地部署应用',
                        required=False, nargs='?', const='default_value')
    args = vars(parser.parse_args())

    if args is None:
        print(echo_help())
        if version != get_cli_server_version():
            print("pip install sipiiiii")
        sys.exit(1)

    if args['deployed']:
        # execute deployed code here
        _, msg = deployed()
        print(msg)

    elif args['dev']:
        # execute dev code here
        _, msg = dev_app()
        print(msg)

    elif args['new']:
        # execute new code here
        status = False
        msg = ""
        if args['new'] == "default_value":
            status, msg, app_name = create_app_project()
        else:
            status, msg, app_name, _ = create_app(args['new'])

        if status:
            print(f"{msg}")
        else:
            print(msg)

    elif args['login']:
        email = input("输入你的email:")
        password = getpass.getpass('输入你的密码:')
        if email == "" or email is None:
            print("没有输入email.")
            sys.exit(1)
        if password == "" or password is None:
            print("没有输入密码.")
            sys.exit(1)

        _, msg = login(email, password)
        print(msg)
    elif args['register']:
        username = input("请输入你的用户名:")
        email = input("输入你的email:")
        password = getpass.getpass('输入你的密码:')
        if username == "" or username is None:
            print("没有输入用户名.")
            sys.exit(1)
        if email == "" or email is None:
            print("没有输入email.")
            sys.exit(1)
        if password == "" or password is None:
            print("没有输入密码.")
            sys.exit(1)
        _, msg = register(username, email, password)

        print(msg)
    elif args['activate']:
        _, msg = activate_account()
        print(msg)

    elif args['list']:
        _, apps = show_list()
        print(apps)
    elif args['tally']:
        status, tally, info = get_tally()
        if not status:
            print(tally)
        else:
            print(f"你的积分: {tally}, {info}")

    elif args['info']:
        status = False
        app_info = ""
        if args['info'] == "default_value":
            status, _, app_info = get_app_info()
        else:
            status, _, app_info = get_app_info(args['info'])

        print(app_info)
    elif args['stop']:
        status = False
        msg = ""
        if args['stop'] == "default_value":
            status, msg = stop_app(None)
        else:
            status, msg = stop_app(args['stop'])

        print(msg)
    elif args['restart']:
        status = False
        msg = ""
        if args['restart'] == "default_value":
            status, msg = restart_app(None)
        else:
            status, msg = restart_app(args['restart'])

        print(msg)

    elif args['localdepl']:
        status = False
        msg = ""
        if args['localdepl'] == "default_value":
            status, msg = localdepl(None)
        else:
            status, msg = localdepl(args['localdepl'])

        print(msg)

    elif args['help']:
        print(echo_help())
    else:
        print(echo_help())
        if version != get_cli_server_version():
            print("\r\npip install sipiiiii -U")


if __name__ == "__main__":
    main()



import os
import yaml
import json
import sys

from shutil import rmtree
from pathlib import Path
from jinja2 import Environment, FileSystemLoader
from .utils.config import fix
from .utils.Tools import get_current_path, read_app_yaml


class ExecAppClass(object):
    def __init__(self,file_path):
        self.file_path = file_path

    def run_python_app(self):
        cmd_line = input("请输出python解析器[python,python3或pypy](默认python):")
        if cmd_line is None or cmd_line == "":
            cmd_line = "python"
        # sys.path.append(self.cwd)
        # try:
        #     from app import app
        # except Exception as e:
        #     return False, f"错误: sipiiii项目路径. {e}"
        os.system(f"{cmd_line} {self.file_path}")
        # app.run(debug=True, port=port)
        return True, "服务器运行正常过，目前服务器已停止..."


class CreateAppTypeClass(object):
    def __init__(self, app_name, app_yaml, app_dir):
        self.app_name = app_name
        self.app_yaml = app_yaml
        self.app_dir = app_dir

    def __create_app_path(self):
        if os.path.exists(self.app_name):
            print(f"{self.app_name} 路径已存在，无法在不删除的情况下创建新项目.")
            is_rm = input("是否需要删除, 不删除则会退出程序(y/n)?")
            is_rm = is_rm.lower()
            if is_rm == "y":
                rmtree(self.app_name)
            elif is_rm == "n":
                return True, "退出"

        path = Path(f"{self.app_name}")
        path.mkdir(parents=True, exist_ok=True)

    def __create_code(self, code_type, app_type, code="", code_name="app"):
        if code_name != "app":
            status, msg = create_app_code(
                self.app_name, code_type, code, code_name)
            if not status:
                return False, f"创建代码出错, {msg}"
            return True, msg

        status, msg = create_framework_code(
            self.app_dir, self.app_name, self.app_yaml, code_type, app_type, code_name)
        if not status:
            return False, f"创建框架代码出错, {msg}"
        return True, msg

    def __create_python_requirements(self, app_type):
        requirements = input("输入导入的库(多个库用逗号分隔):")
        if requirements is None or requirements == "":
            requirements = ""

        if requirements.find(",") > -1:
            requirements = requirements.split(",")
        else:
            if requirements == "":
                requirements = []
            else:
                requirements = [requirements]

        if app_type == "fastapi":
            requirements.append('fastapi')
        if app_type == "flask":
            requirements.append('flask')
        if app_type == "openai":
            requirements.append('quart')
            requirements.append('quart_cors')
        if app_type == "streamlit":
            requirements.append('streamlit')

        status, requirements_file_path = create_requirements(
            self.app_name, requirements)
        if not status:
            return False, requirements_file_path

    def create_streamlit_app(self, code_type):
        self.__create_app_path()
        if code_type == "python":
            self.__create_python_requirements("streamlit")

        status, openapi_file_path = create_app_yaml(
            self.app_name, self.app_yaml, "streamlit")
        if not status:
            return False, openapi_file_path

        code = f"""
import streamlit as st

def main():
    st.markdown("hello world!")

if __name__ == "__main__":
    st.set_page_config(
        page_title="{self.app_name}", page_icon=":pencil2:"
    )
    st.title("{self.app_name}")
    # st.sidebar.subheader("Configuration")
    main()
        """
        ststus, msg = self.__create_code(
            code_type, "streamlit", code=code, code_name="main")
        if not ststus:
            return False, msg

        ststus, msg = self.__create_code(code_type, "streamlit")
        if not ststus:
            return False, msg
        return True, f"创建streamlit框架代码成功, 项目目录: ./{self.app_name}"

    def create_flask_app(self, code_type):
        self.__create_app_path()

        if code_type == "python":
            self.__create_python_requirements("flask")

        status, openapi_file_path = create_app_yaml(
            self.app_name, self.app_yaml, "flask")
        if not status:
            return False, openapi_file_path

        ststus, msg = self.__create_code(code_type, "flask")
        if not ststus:
            return False, msg

        return True, f"创建flask框架代码成功, 项目目录: ./{self.app_name}"

    def create_fastapi_app(self, code_type):
        self.__create_app_path()

        if code_type == "python":
            self.__create_python_requirements("fastapi")

        status, openapi_file_path = create_app_yaml(
            self.app_name, self.app_yaml, "fastapi")
        if not status:
            return False, openapi_file_path

        ststus, msg = self.__create_code(code_type, "fastapi")
        if not ststus:
            return False, msg

        return True, f"创建fastapi框架代码成功, 项目目录: ./{self.app_name}"

    def create_openai_app(self, code_type):
        ai_plugin_json = {
            "schema_version": "",
            "name_for_human": "",
            "name_for_model": "",
            "description_for_human": "",
            "description_for_model": "",
            "auth": {
                "type": "none"
            },
            "api": {
                "type": "openapi",
                "url": "!HOSTNAME!/openapi.yaml",
                "is_user_authenticated": False
            },
            "logo_url": "!HOSTNAME!/logo.png",
            "contact_email": "support@example.com",
            "legal_info_url": "https://example.com/legal"
        }

        try:
            ai_plugin_json['schema_version'] = self.app_yaml.get(
                'info', {}).get('version')
            ai_plugin_json['name_for_human'] = self.app_yaml.get(
                'info', {}).get('title')
            ai_plugin_json['name_for_model'] = self.app_yaml.get(
                'info', {}).get('title')
            ai_plugin_json['description_for_human'] = self.app_yaml.get(
                'info', {}).get('description')
            ai_plugin_json['description_for_model'] = self.app_yaml.get(
                'info', {}).get('description')
        except Exception as e:
            return False, str(e)

        self.__create_app_path()

        if code_type == "python":
            self.__create_python_requirements("openai")

        status, openapi_file_path = create_app_yaml(
            self.app_name, self.app_yaml, "openai")
        if not status:
            return False, openapi_file_path

        status, plugin_json_file_path = create_app_config(
            self.app_name, ai_plugin_json, "openai")
        if not status:
            return False, plugin_json_file_path

        ststus, msg = self.__create_code(code_type, "openai")
        if not ststus:
            return False, msg

        return True, f"创建openai框架代码成功, 项目目录: ./{self.app_name}"


def get_app_name(app_name, is_yaml=False):
    if app_name is None:
        while True:
            app_yaml_file = ""
            try:
                app_yaml_file = input("请输入APP yaml文件:")
            except KeyboardInterrupt:
                print("\r\n")
                sys.exit(1)

            if app_yaml_file == "":
                continue
            status, app_name, openapi_yaml, aiapi_dir = yaml_app_info(
                app_yaml_file, None)
            if not status:
                return False, app_name, openapi_yaml, aiapi_dir

            return True, app_name, openapi_yaml, aiapi_dir

    elif app_name.find(".yaml") > -1:
        status, app_name, openapi_yaml, aiapi_dir = yaml_app_info(
            app_name, None)
        if not status:
            return False, app_name, openapi_yaml, aiapi_dir
        return True, app_name, openapi_yaml, aiapi_dir
    else:
        if is_yaml:
            return False, app_name, "", ""
        return True, app_name, "", ""


def get_app_code_type(file_path):
    _fix = ""
    try:
        _fix = os.path.splitext(file_path)[-1].split(".")[-1]
    except Exception as e:
        return False, f"错误: {e}"

    if _fix not in fix.values():
        return False, f"{_fix} 不存在于 {list(fix.keys())}"

    new_fix = {v: k for k, v in fix.items()}
    return True, new_fix[_fix]


def yaml_app_info(yaml_file, app_type):
    openapi_yaml = None
    aiapi_dir = get_current_path()
    # if app_type != "openai":
    #     app_type = "other"
    if yaml_file is None:
        openapi_yaml = read_app_yaml(aiapi_dir, app_type)
    else:
        openapi_yaml = read_app_yaml(yaml_file, app_type)
    if openapi_yaml == "":
        return False, "没找到yaml文件.", None, None
    app_name = ""
    try:
        app_name = openapi_yaml['info']['title']
    except Exception as e:
        return False, str(e), None, None

    if app_name == "":
        return False, "yaml中没有找到 yaml[info][title]", None, None

    app_name = app_name.replace(" ", "")
    return True, app_name, openapi_yaml, aiapi_dir


def create_requirements(directory_path, requirements):
    if type(requirements) != list:
        return False, "requirements 错误!"

    if not os.path.exists(directory_path):
        return False, f"{directory_path} 没有找到."

    requirements_file_path = os.path.join(directory_path, "requirements.txt")
    requirements_data = ""
    if len(requirements) > 0:
        requirements_data = "\n".join(requirements)

    with open(requirements_file_path, "w") as f:
        f.write(requirements_data)

    return True, requirements_file_path


def create_app_config(directory_path, plugin_json, app_type):
    if type(plugin_json) != dict:
        return False, "应用配置文件错误!"

    if not os.path.exists(directory_path):
        return False, f"{directory_path} 目录没有找到."

    # ai-plugin.json
    if app_type == "openai":
        app_type = "ai-plugin"
    else:
        app_type = "config"

    plugin_json_file_path = os.path.join(directory_path, f"{app_type}.json")
    # plugin_json_data = "\n".join(plugin_json_file_path)
    with open(plugin_json_file_path, encoding="utf-8", mode="w") as f:
        json.dump(plugin_json, f, indent=2)

    return True, plugin_json_file_path


def create_app_yaml(directory_path, openapi_yaml, app_type):
    if not os.path.exists(directory_path):
        return False, f"{directory_path} 目录不存在"

    # openapi_yaml_file_path = os.path.join(directory_path, f"{app_type}.yaml")
    # if app_type is None:
    openapi_yaml_file_path = os.path.join(directory_path, "aiapi.yaml")
    # openapi.yaml
    # openapi_yaml_file_path = os.path.join(directory_path, f"{app_type}.yaml")

    # plugin_json_data = "\n".join(plugin_json_file_path)
    with open(openapi_yaml_file_path, encoding="utf-8", mode="w") as f:
        yaml.dump(openapi_yaml, stream=f, allow_unicode=True)

    return True, openapi_yaml_file_path


def create_app_code(app_path, code_type, code, code_name):
    _fix = fix.get(code_type, None)
    if _fix is None:
        return False, "没有获取到应用类型"

    app_file = os.path.join(app_path, f"{code_name}.{_fix}")
    with open(app_file, 'w', encoding="utf-8") as f:
        f.write(code)

    return True, "创建代码成功"


def create_framework_code(directory_path, app_path, openapi_yaml, code_type, app_type, code_name="app"):
    if openapi_yaml.get("paths", None) is None:
        openapi_yaml['paths'] = []
    env = Environment(loader=FileSystemLoader(f"{directory_path}"))
    # template.j2
    # if app_type != "openai":
    #     app_type = "other"
    j2_file = f"{code_type}_{app_type}"
    try:
        template = env.get_template(f'{j2_file}.j2')
        output = template.render(paths=openapi_yaml['paths'])
    except Exception as e:
        return False, str(e)
    _fix = fix.get(code_type, None)
    if _fix is None:
        return False, "没有获取到应用类型"
    # Write the output to a Python file
    app_file = os.path.join(app_path, f"{code_name}.{_fix}")
    with open(app_file, 'w', encoding="utf-8") as f:
        f.write(output)

    return True, "创建框架代码成功"



import json
import jsonpath
import xmltodict


# 把dict['xxx']转换成dict.xxx
class DottableDict(dict):
    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)
        self.__dict__ = self

    def allowDotting(self, state=True):
        if state:
            self.__dict__ = self
        else:
            self.__dict__ = dict()


def str_to_json(data):
    return json.loads(data)


def json_to_str(data):
    if verify_json(data) is True:
        return json.dumps(data)

    return ""


def search_json(data, search, node="$"):
    ret = jsonpath.jsonpath(data, f"{node}..{search}")
    if ret is False:
        ret = []
    return ret


def verify_json(data):
    json_str = ""
    try:
        json_str = json.dumps(data)
    except Exception as _:
        pass
    try:
        json.loads(json_str)
    except ValueError:
        return False

    return True


def json_to_xml(jsonstr):
    # xmltodict库的unparse()json转xml
    xmlstr = xmltodict.unparse(jsonstr)
    return xmlstr


def xml_to_json(xmlstr):
    # parse是的xml解析器
    xmlparse = xmltodict.parse(xmlstr)
    # json库dumps()是将dict转化成json格式，loads()是将json转化成dict格式。
    # dumps()方法的ident=1，格式化json
    jsonstr = json.dumps(xmlparse, indent=1)
    return jsonstr



import zipfile
import tempfile
import os


def zip_folder(folder_path):
    # 创建一个临时文件来保存zip文件
    zip_file_path = None
    with tempfile.NamedTemporaryFile(suffix='.zip', delete=False) as tf:
        zip_file_path = tf.name
    if zip_file_path is None:
        return None

    # 压缩文件夹
    with zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED) as zf:
        for root, dirs, files in os.walk(folder_path):
            for file in files:
                # 构造文件的绝对路径
                file_path = os.path.join(root, file)
                # 将文件添加到zip文件中
                zf.write(file_path, os.path.relpath(file_path, folder_path))

    return zip_file_path



# coding: utf-8
try:
    from utils.formatConversion import verify_json, DottableDict
except ImportError:
    from .formatConversion import verify_json, DottableDict

import validators
import httpx
import time


def log_request(request):
    # print(
    #     f"Request event hook: {request.method} {request.url} - Waiting for response")
    pass


def log_response(response):
    request = response.request
    # print(
    #     f"Response event hook: {request.method} {request.url} - Status {response.status_code}"
    # )
    pass


def raise_on_4xx_5xx(response):
    response.raise_for_status()


class HTTP(object):
    def __init__(self, proxies=None):
        pass
        self._proxy = proxies
        if proxies is not None:
            if verify_json(proxies) is True:
                self._proxy = proxies
            else:
                raise Exception(
                    f"proxies not json , header value now :{proxies}")
        self._verify = False
        try:
            self.session = httpx.Client(
                event_hooks={
                    "request": [log_request],
                    "response": [log_response, raise_on_4xx_5xx],
                },
                proxies=self._proxy,
                verify=self._verify
            )
        except Exception as e:
            print(e)

        self._cookie = httpx.Cookies()
        self._timeout = 60
        self._verify = False
        # self._ret_data = {}  # DottableDict()
        # self._ret_data.allowDotting()
        self._rep_num = 3
        self._rep_sleep = 1

    # @property
    # def ret_data(self):
    #     return self._ret_data

    # @ret_data.getter
    # def ret_data(self):
    #     return self._ret_data

    @property
    def verify(self):
        return self._verify

    @verify.setter
    def verify(self, value):
        if type(value) == bool:
            self._verify = value
        else:
            self._verify = False

    @verify.getter
    def verify(self):
        return self._verify

    @property
    def headers(self):
        return self._header

    @headers.setter
    def headers(self, value):
        if verify_json(value) is True:
            self._header = value
        else:
            raise Exception(
                f"header not json , header value now :{self._header}")

    @headers.getter
    def headers(self):
        return self._header

    @property
    def cookie(self):
        return self._cookie

    @cookie.setter
    def cookie(self, value):
        try:
            value["name"]
            value["value"]
        except Exception as e:
            raise e

        try:
            value["domain"]
        except Exception:
            value["domain"] = ""
        try:
            value["path"]
        except Exception:
            value["path"] = "/"
        self._cookie.set(value["name"], value["value"],
                         value["domain"], value["path"])

    @cookie.getter
    def cookie(self):
        return self._cookie

    @property
    def timeout(self):
        return self._timeout

    @timeout.setter
    def timeout(self, value):
        if type(value) != int:
            try:
                value = int(value)
            except Exception as e:
                raise e

        self._timeout = value

    @timeout.getter
    def timeout(self):
        return self._timeout

    @property
    def rep_num(self):
        return self._rep_num

    @rep_num.setter
    def rep_num(self, value):
        if type(value) == int:
            self._rep_num = value
        else:
            try:
                self._rep_num = int(self._rep_num)
            except ValueError:
                self._rep_num = 3

    @rep_num.getter
    def rep_num(self):
        return self._rep_num

    @property
    def rep_sleep(self):
        return self._rep_sleep

    @rep_sleep.setter
    def rep_sleep(self, value):
        if type(value) == int:
            self._rep_sleep = value
        else:
            try:
                self._rep_sleep = int(self._rep_sleep)
            except ValueError:
                self._rep_sleep = 1

    @rep_sleep.getter
    def rep_sleep(self):
        return self._rep_sleep

    def req(self, req_type="POST", url="", body=None, json_data=None, params_data=None, files_data=None):
        if self._header == {}:
            return {
                "code": 500,
                "success": False,
                "msg": "header头没有指定",
                "data": {}
            }
        if validators.url(url) is not True:
            return {
                "code": 500,
                "success": False,
                "msg": "url没有指定",
                "data": {}
            }
        # self.session.timeout(self._timeout)

        if json_data is not None:
            if verify_json(json_data) is False:
                return {
                    "code": 500,
                    "success": False,
                    "msg": "参数错误",
                    "data": {}
                }
                #raise Exception(f"not json, json_data value now :{json_data}")
        resp = None

        for _ in range(self._rep_num):
            try:
                resp = self.session.request(
                    method=req_type.lower(),
                    url=url,
                    data=body,
                    json=json_data,
                    params=params_data,
                    files=files_data,
                    headers=self._header,
                    cookies=self._cookie,
                    timeout=self._timeout,
                )
                resp.encoding = "UTF-8"
                if resp.status_code in [200, 201, 202, 203, 204, 205, 206, 207, 300, 301, 302, 303, 304, 305, 306, 307]:
                    break
                time.sleep(self._rep_sleep)

            except Exception as e:
                return {
                    "code": 500,
                    "success": False,
                    "msg": f"连接错误: {e}",
                    "data": {}
                }

        try:
            json_data = resp.json()
        except Exception as e:
            print(e)
            json_data = {}

        # self._ret_data.code = resp.status_code
        # self._ret_data.json = json_data

        app_ret_json = {
            "code": resp.status_code,
            "success": json_data.get("success", False),
            "msg": json_data.get("msg", "请求失败"),
            "data": json_data,
        }

        return app_ret_json

    def __del__(self):
        try:
            if self.session is not None:
                self.session.close()
        except Exception as _:
            pass


HTTPS = HTTP


def https_get(url, params=None, headers={}):
    https = HTTPS()
    https.headers = headers
    res = https.req("get", url=url, params_data=params)
    del https
    res_class = DottableDict(res)
    res_class.allowDotting()
    return res_class


def https_post(url, json_data=None, headers={}):
    https = HTTPS()
    headers['accept'] = 'application/json'
    https.headers = headers
    https.timeout = 60
    res = https.req("post", url=url, json_data=json_data)
    del https
    res_class = DottableDict(res)
    res_class.allowDotting()
    return res_class


def https_file(url, files=None, headers={}):
    import requests
    #https = HTTPS()
    #https.headers = headers
    headers['Content-type'] = files.content_type
    res = requests.post(url, data=files, headers=headers, timeout=300)
    # res = https.req("post", url=url, json_data={}, files_data=files)
    res_data = {
        "code": res.status_code,
        "success": res.json()["success"],
        "msg": res.json()["msg"],
        "data": res.json()
    }
    del res
    res_class = DottableDict(res_data)
    res_class.allowDotting()
    return res_class




import setuptools
import sys
import os
from shutil import copyfile
from pathlib import Path

# from version import version
# from initsipiiiii import init_framework


def get_current_path():
    aiapi_dir = os.path.join(Path.home(), ".aiapi")
    if os.path.exists(aiapi_dir):
        return aiapi_dir
    else:
        path = Path(f"{aiapi_dir}")
        path.mkdir(parents=True, exist_ok=True)
        return aiapi_dir


def init_framework():
    aiapi_dir = get_current_path()

    # Replace 'template/template.yaml' with the path to the file you want to copy
    # src_file = f'sipiiiii{os.sep}template/template.yaml'
    # try:
    src_dir = os.path.join(os.getcwd(), "template")
    # src_dir = os.path.join('sipiiiii', "template")
    try:
        template_files = os.listdir(src_dir)
    except Exception as e:
        print(e)
        src_dir = os.path.join(os.getcwd(), "sipiiiii", "template")
        template_files = os.listdir(src_dir)

    for tFIle in template_files:
        src_file = os.path.join(src_dir, tFIle)

        # Use Path().resolve() to get the absolute path of the source file
        src_path = Path(src_file).resolve()

        # Use Path().joinpath() to get the destination path
        dest_path = Path(aiapi_dir).joinpath(tFIle)
        # Use Path().replace() to copy the file
        try:
            copyfile(src_path, dest_path)
        except Exception as e:
            print(str(e))
            continue

        # Check if the file was copied successfully
        if not dest_path.exists():
            continue

    return True, "初始化成功"


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

status, msg = init_framework()
if not status:
    print(msg)
    sys.exit(1)

setuptools.setup(
    # 包的分发名称，使用字母、数字、_、-
    name="sipiiiii",
    # 版本号, 版本号规范：https://www.python.org/dev/peps/pep-0440/
    version="1.0.0",
    # 作者名
    author="sipiiiii@admin",
    # 作者邮箱
    author_email="j0hn.wahahaha@gmail.com",
    # 包的简介描述
    description="depl sipiiiii, App Development Framework",
    # 包的详细介绍(一般通过加载README.md)
    long_description=long_description,
    # 和上条命令配合使用，声明加载的是markdown文件
    long_description_content_type="text/markdown",
    # 项目开源地址
    url="http://www.depl.run/",
    # 如果项目由多个文件组成，我们可以使用find_packages()自动发现所有包和子包，而不是手动列出每个包，在这种情况下，包列表将是example_pkg
    packages=setuptools.find_packages(),
    package_data={'template': ['*.j2', '*.yaml']},
    nclude_package_data=True,
    install_requires=[
        'httpx',
        'prettytable==3.7.0',
        'jinja2',
        'requests',
        'requests_toolbelt',
        'xmltodict',
        'validators',
        'pathlib',
        'jsonpath',
        'pyyaml'
    ],
    entry_points={
        'console_scripts': [
            'sipiiiii = sipiiiii.main:main'
        ]
    },
    # 关于包的其他元数据(metadata)
    classifiers=[
        # 该软件包仅与Python3兼容
        "Programming Language :: Python :: 3",
        # 根据MIT许可证开源
        "License :: OSI Approved :: MIT License",
        # 与操作系统无关
        "Operating System :: OS Independent",
    ],
)



import os
import multiprocessing
import prettytable as pt
import sys
import time
import yaml

from requests_toolbelt.multipart.encoder import MultipartEncoder
from pathlib import Path

from .core import CreateAppTypeClass, yaml_app_info, ExecAppClass, get_app_code_type, get_app_name

from .utils.ZipFileTool import zip_folder
from .utils.HttpSDK import https_get, https_post, https_file
from .utils.Tools import get_outside_ip, is_encomm_char, write_new_yaml
from .utils.config import app_all_types, server_host, fix


def process_update_outside_ip(domain, _token):
    while True:
        time.sleep(60 * 30)
        outside_ip = get_outside_ip()
        headers = {
            "Content-type": "application/json",
            "authorization": _token
        }
        url = f"{server_host}/system/openapi/modify/domain/bind/ip"
        data = {
            "domain": domain,
            "ip": outside_ip
        }
        response = https_post(url, json_data=data, headers=headers)
        if not response.success:
            print(f"更新域名错误: {response.msg}")
        else:
            print(f"域名IP已更新: {domain} -> {outside_ip}")


def get_token():
    _token = ""
    home_dir_file = os.path.join(Path.home(), ".sipitoken")
    if os.path.exists(home_dir_file):
        with open(home_dir_file, 'r', encoding='utf8') as f:
            _token = f.read()
    if _token == "":
        return False, "请登录 sipiiiii, 使用 sipiiiii -L 进行登录, 使用 sipiiiii -h 获得帮助"

    return True, _token


# 创建项目
def create_app_project():
    status, _token = get_token()
    if not status:
        return False, _token, ""

    print("""
    *** 应用名称必须英文
    *** 可以使用向导模式创建应用和使用模版创建应用
        """)

    while True:
        is_wizard_create = input("y使用向导创建应用,n使用模版生成(y/n):")
        is_wizard_create = is_wizard_create.lower()

        if is_wizard_create == "y":
            app_name = input("请输入应用名:")
            if app_name == "" or not is_encomm_char(app_name):
                print("应用名不能为空, 并且应用名只能使用英文字母")
                continue

            app_desc = input("请输入应用说明:")
            app_version = input("请输入应用版本(默认 v1.0):")
            if app_version == "":
                app_version = "v1.0"
            print(f"应用名: {app_name}, 应用说明: {app_desc}, 应用版本: {app_version}")

            status, yaml_data = create_yaml(
                app_name, app_version, app_desc)
            if not status:
                print(yaml_data)
                continue

            with open(f"{os.getcwd()}{os.sep}aiapi.yaml", "w", encoding="utf-8") as f:
                f.write(yaml_data)

            status, msg, app_name, app_type = create_app(
                f"{os.getcwd()}{os.sep}aiapi.yaml")
            if status:
                write_new_yaml(
                    f"{os.getcwd()}{os.sep}{app_name}{os.sep}aiapi.yaml", app_type)

            os.remove(f"{os.getcwd()}{os.sep}aiapi.yaml")
            break

        elif is_wizard_create == "n":
            status, msg, app_name, _ = create_app(None)
            break
        else:
            continue

    return status, msg, app_name


# 创建yaml模版
def create_yaml(app_name, app_version, app_desc):
    status, _token = get_token()
    if not status:
        return False, _token

    while True:
        route_num = input("请输入有多少个接口(默认1个接口):")
        if route_num == "":
            route_num = 1

        try:
            route_num = int(route_num)
            break
        except Exception as _:
            print("接口数量必须为数字")

    route_json = {
        "swagger": "2.0",
        "info": {
            "title": app_name,
            "description": app_desc,
            "version": app_version
        },
        "paths": {}
    }
    for i in range(route_num):
        i = i + 1
        api_method = input(f"第{i}个接口的请求方式(get, post, put, delete):")
        api_name = input(f"第{i}个接口的名字,请用英文:")
        api_dec = input(f"第{i}个接口的描述:")
        parameters_end = False
        params_list = []
        while parameters_end:
            params_name = input("请输入参数名:")
            params_type = input("请输入参数类型(float, int, string):")
            params_list.append({
                "name": params_name,
                "in": "query",
                "schema": {
                    "type": params_type
                }
            })
            parameters_end = input("是否继续录入参数(y/n):")
            parameters_end = parameters_end.lower()
            if parameters_end == "y":
                parameters_end = True
            else:
                parameters_end = False
        route_json['paths'][f"/{api_name}"] = {
            api_method: {
                "summary": api_dec,
                # "parameters": params_list,
                "responses": {
                    "200": "OK"
                }
            }
        }
    yaml_data = yaml.dump(route_json, allow_unicode=True)

    return True, yaml_data

# 通过yaml直接创建APP


def create_app(yaml_file=None):
    if yaml_file is not None and yaml_file.find(".yaml") == -1:
        return False, "参数错误, 参数并不是Yaml文件, 请查看 sipiiiii -h", "", ""
    status, _token = get_token()
    if not status:
        return False, _token, "", ""

    while True:
        code_type = input(
            f"请输入创建编程语言{list(fix.keys())}(默认第一个): "
        )

        if code_type == "":
            code_type = list(fix.keys())[0]

        _code_type = app_all_types.get(code_type, None)
        if _code_type is None:
            print(f"编程语言的范围为: {list(fix.keys())}")
            continue
        break

    while True:
        app_type = input(
            f"请创建应用程序类型{app_all_types[code_type]}(默认第一个): "
        )
        if app_type == "":
            app_type = app_all_types[code_type][0]
        if app_type not in app_all_types[code_type]:
            print(f"应用程序类型的范围为: {app_all_types[code_type]}")
            continue
        break

    status, app_name, app_yaml, app_dir = yaml_app_info(yaml_file, app_type)
    if not status:
        return False, f"创建应用失败: {app_name}", "", ""

    catc = CreateAppTypeClass(app_name, app_yaml, app_dir)
    method_of_catc = getattr(catc, f"create_{app_type}_app")
    status, msg = method_of_catc(code_type)
    return status, msg, app_name, app_type


def dev_app():
    print("测试运行....\r\n")
    run_main = input("请输入APP运行文件:")
    cwd = os.getcwd()

    file_path = os.path.join(cwd, run_main)
    file_path = Path(f'{file_path}')
    if not file_path.exists():
        return False, "错误: 不存在项目路径, 请到项目路径下运行"

    status, code_type = get_app_code_type(file_path)
    if not status:
        return False, code_type

    eac = ExecAppClass(file_path)

    method_of_catc = getattr(eac, f"run_{code_type}_app")
    status, msg = method_of_catc()

    return status, msg


def localdepl(yaml_file_name):
    status, _token = get_token()
    if not status:
        return False, _token

    status, app_name, app_yaml, _ = get_app_name(yaml_file_name, True)
    if not status:
        return False, f"获取应用名错误: {app_name}"

    app_cn_name = input("请输入应用别名:")
    if app_cn_name == "":
        app_cn_name = app_name

    print(f"本地部署: {app_name} 项目\r\n")

    description = ""
    try:
        description = app_yaml.get(
            'info', {}).get('description', "")
    except Exception as e:
        return False, str(e)

    outside_ip = get_outside_ip()
    if outside_ip == "":
        return False, "无法访问互联网"

    print(f"本机外网IP: {outside_ip}")
    status, _token = get_token()
    if not status:
        return False, _token

    run_main = input("请输入APP运行文件:")

    if run_main.find(".") == 0:
        return False, "APP运行文件必须是文件名加文件后缀"

    run_main_fix = run_main.split(".")
    if len(run_main_fix) != 2:
        return False, "APP运行文件输入不合法"

    if run_main_fix[-1] not in list(fix.values()):
        return False, f"APP运行文件不在允许的列表范围内: {list(fix.keys())}"

    cwd = os.getcwd()
    file_path = os.path.join(cwd, run_main)
    file_path = Path(f'{file_path}')
    if not file_path.exists():
        return False, "错误: 不存在项目路径, 请到项目路径下运行"

    app_language = [k for k, v in fix.items() if v == run_main_fix[-1]][0]

    url = f"{server_host}/system/openapi/app/local/deployment"
    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }
    data = {
        "app_name": app_name,
        "app_dock": "app",
        "app_cn_name": app_cn_name,
        "ip": outside_ip,
        "description": description,
        "app_language": app_language
    }

    response = https_post(url, json_data=data, headers=headers)
    if response.code != 200:
        return False, "内部服务器错误，请稍后再试"

    if not response.success:
        return False, response.msg

    p = multiprocessing.Process(
        target=process_update_outside_ip, args=(response.data['domain'], _token), daemon=True)
    p.start()

    status, code_type = get_app_code_type(file_path)
    if not status:
        return False, code_type

    eac = ExecAppClass(file_path)

    method_of_catc = getattr(eac, f"run_{code_type}_app")
    status, msg = method_of_catc()

    return status, msg


def stop_app(app_name=None):
    status, _token = get_token()
    if not status:
        return False, _token

    status, app_name, _, _ = get_app_name(app_name)
    if not status:
        return False, f"获取应用名错误: {app_name}"

    data = {"app_name": app_name}

    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }
    url = f"{server_host}/system/openapi/app/stop"
    response = https_post(url, json_data=data, headers=headers)
    if not response.success:
        return False, response.msg

    return True, response.msg


def deployed():
    status, _token = get_token()
    if not status:
        return False, _token
    print("""
    *** 必须在项目目录下进行操作
    *** 暴露的端口必须为8000
    """)
    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }

    url = f"{server_host}/system/openapi/users/check/token"
    # response = requests.get(url,headers=headers, timeout=20)
    response = https_get(url, headers=headers)

    if not response.success:
        return False, "用户令牌无效，请登录使用 sipiiiiii -L 登录."

    if not response.data["exist"]:
        return False, "用户令牌无效或超时，请登录使用 sipiiiiii -L 登录."

    app_cn_name = input("请输入APP别名:")
    # input("please input APP docking party(chatgpt, other):")
    app_dock = "other"
    run_main = input("请输入APP运行文件:")

    if app_cn_name == "" or run_main == "":
        return False, "APP别名 或者 APP运行文件的不能为空."

    if run_main.find(".") == 0:
        return False, "APP运行文件必须是文件名加文件后缀"

    run_main_fix = run_main.split(".")
    if len(run_main_fix) != 2:
        return False, "APP运行文件输入不合法"

    cwd = os.getcwd()
    file_path = os.path.join(cwd, run_main)
    file_path = Path(f'{file_path}')
    if not file_path.exists():
        return False, "错误: 不存在项目路径, 请到项目路径下运行, 或者使用sipiiiii -n 创建项目"

    if run_main_fix[-1] not in list(fix.values()):
        return False, f"APP运行文件不在允许的列表范围内: {list(fix.keys())}"

    app_language = [k for k, v in fix.items() if v == run_main_fix[-1]][0]

    zip_file_path = zip_folder(cwd)
    if zip_file_path is None:
        return False, "应用程序打包失败。请检查应用程序目录中是否存在异常文件或文件夹"

    filename = os.path.basename(cwd)
    print(f"远程部署: {filename} 项目\r\n")
    # filename = ""#os.path.basename(zip_file_path)
    files = MultipartEncoder(fields={
        'file': (f"{filename}.zip", open(zip_file_path, 'rb'), 'application/octet-stream'),
    })

    url = f"{server_host}/system/openapi/app/upload"
    response = https_file(url, files=files, headers=headers)
    # response = requests.post(url, data=files, headers=headers, timeout=300)
    # print(response)
    if response['code'] != 200:
        return False, "内部服务器错误，请稍后再试"

    if not response['success']:
        return False, response['msg']

    url = f"{server_host}/system/openapi/app/deployment"
    data = {
        "file_uuid": response['data']['file_uuid'],
        "app_cn_name": app_cn_name,
        "app_language": app_language,
        "app_dock": app_dock,
        "run_main": run_main
    }
    headers['Content-type'] = "application/json"
    # headers['accept'] = 'application/json'

    response = https_post(url, json_data=data, headers=headers)
    # response = requests.post(url, data=files, headers=headers, timeout=300)
    # print(response)
    if response.code != 200:
        return False, "内部服务器错误，请稍后再试"

    if not response.success:
        return False, response.msg

    # return True, "ok"

    # status, info_json, _ = get_app_info(filename)
    # if not status:
    #     return False, "部署成功，状态获取失败"

    # bar = ProgPercent(50, monitor=True)
    print("正在远程部署...\r\n")
    i = 0
    while True:
        if i >= 60 * 5:
            return False, "\r\n获取状态超时, 请用 sipiiiii -l查看应用列表"

        status, info_json, _ = get_app_info(filename)
        if not status:
            # bar.update(50)
            # return False, "\r\n部署成功，状态获取失败"
            continue

        if info_json['ServerStatus'] == "close" or info_json['ServerStatus'] == "exited":
            # bar.update(50)
            return True, f"\r\n应用程序部署出错，日志: \r\n{info_json['Log']}"

        if info_json['ServerStatus'] == "created" or info_json['ServerStatus'] == "running":
            # bar.update(50)
            return True, f"\r\n部署成功, \r\n域名: {info_json['Domain']}"

        i += 1
        time.sleep(1)

    # return True, response.data


def login(email, password):
    url = f"{server_host}/system/openapi/users/login"
    headers = {
        "Content-type": "application/json"
    }
    data = {
        "email": email,
        "password": password
    }
    # response = requests.post(url, json=data, headers=headers, timeout=20)
    response = https_post(url, json_data=data, headers=headers)
    if response.code != 200:
        return False, "服务器错误."
    if not response.success:
        return False, response.msg

    home_dir_file = os.path.join(Path.home(), ".sipitoken")
    with open(home_dir_file, 'w', encoding='utf-8') as f:
        f.write(response.data['token'])

    status, tally, info = get_tally()
    if status:
        return True, f"登录成功, 登录有效期3天, 剩余积分: {tally}, {info}"

    return False, "登录成功, 登录有效期3天, 获取积分失败."


def get_tally():
    status, _token = get_token()
    if not status:
        return False, _token

    url = f"{server_host}/system/openapi/users/tally"
    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }
    response = https_get(url, headers=headers)
    # response = requests.get(url, headers=headers, timeout=20)
    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试", ""

    if not response.success:
        return False, response.msg, ""

    tally = response.data['tally']
    tally = int(tally)
    info = f"可以部署{int(tally/20)}个远程应用或{int(tally/5)}个本地应用"
    return True, response.data['tally'], info


def restart_app(app_name):
    status, _token = get_token()
    if not status:
        return False, _token

    url = f"{server_host}/system/openapi/app/restart"
    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }
    status, app_name, _, _ = get_app_name(app_name)
    if not status:
        return False, f"获取应用名错误: {app_name}"

    response = https_post(
        url, json_data={"app_name": app_name}, headers=headers)
    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, response.msg

    return True, response.msg


def get_app_info(app_name=None):
    status, _token = get_token()
    if not status:
        return False, {}, _token

    url = f"{server_host}/system/openapi/app/status"
    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }
    status, app_name, _, _ = get_app_name(app_name)
    if not status:
        return False, {}, f"获取应用名错误: {app_name}"

    data = {"app_name": app_name}

    response = https_post(url, json_data=data, headers=headers)
    if response.code != 200:
        return False, {}, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, {}, response.msg

    app_info = response.data
    app_info = app_info['data']
    app_info_str = f"""
应用名称       : {app_info['AppName']}
应用别名       : {app_info['AppCnName']}
应用启动文件   : {app_info['AppRun']}
域名           : {app_info['Domain']}
应用状态       : {app_info['ServerStatus']}
创建时间       : {app_info['CreateTime']}
"""
    return True, app_info, app_info_str


def show_list():
    status, _token = get_token()
    if not status:
        return False, _token

    url = f"{server_host}/system/openapi/app/list"
    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }

    #response = requests.get(url, headers=headers, timeout=20)
    response = https_get(url, headers=headers)
    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, response.msg

    app_list = response.data['data']
    table = pt.PrettyTable(
        ['应用名', '应用别名', '域名', '应用状态', '创建时间'])
    for app in app_list:
        table.add_row((app['AppName'], app['AppCnName'], app['Domain'],
                      app['ServerStatus'], app['CreateTime']))

    return True, table


def get_cli_server_version():
    return "1.0.0"


def update_template():
    pass


def activate_account():
    status, _token = get_token()
    if not status:
        return False, _token

    headers = {
        "Content-type": "application/json",
        "authorization": _token
    }

    url = f"{server_host}/system/openapi/user/verify/code"

    response = https_get(url, headers=headers)
    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, f"获取验证码异常: {response.msg}"

    verify_code = ""
    while True:
        try:
            verify_code = input("请输出邮件中的 6位 验证码:")
            verify_code = int(verify_code)
        except KeyboardInterrupt:
            print("\r\n")
            sys.exit(1)
        except ValueError:
            print("验证码为纯6位数字")
            continue
        if verify_code == "":
            continue
        break

    url = f"{server_host}/system/openapi/user/verify/code"

    response = https_post(
        url, json_data={"code": verify_code}, headers=headers)
    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, f"获取验证码异常: {response.msg}"

    return True, response.msg


def register(username, email, password):
    headers = {
        "Content-type": "application/json",
    }
    url = f"{server_host}/system/openapi/user/register"

    data = {
        "username": username,
        "password": password,
        "email": email
    }

    response = https_post(url, json_data=data, headers=headers)

    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, f"注册异常: {response.msg}"

    print(f"用户: {username} 注册成功, 注册后需要接受邮件验证码并且输入验证码后才能使用")
    print("验证码也可以之后使用 -V 进行重新验证")

    home_dir_file = os.path.join(Path.home(), ".sipitoken")
    with open(home_dir_file, 'w', encoding='utf-8') as f:
        f.write(response.data['token'])

    headers["authorization"] = response.data['token']
    url = f"{server_host}/system/openapi/user/verify/code"

    response = https_get(url, headers=headers)
    if response.code != 200:
        return False, "服务器内部错误, 请稍后再试"

    if not response.success:
        return False, f"获取验证码异常: {response.msg}"

    ststus, msg = activate_account()

    return ststus, msg
